from urllib.parse import urlencode
from flask import jsonify, request


def details_view(model, unique_id):
    """
    Generic helper function for rendering details API

    :param(model.Model) model: Model object of the respective entity
    :param(str) unique_id: Unique ID of the resource to be fetched
    :return: JSON object and HTTP Status Code
    """
    record = model.fetch_record(unique_id)
    if not record:
        response = {
            'code': 'NOT FOUND',
            'message': 'Resource with ID {} not found in our database'.format(unique_id)
        }
        status_code = 404
    else:
        response = {
            'data': record
        }
        status_code = 200

    return jsonify(response), status_code


def list_view(model):
    """
    Helper function for list view APIs

    :param(model.Model) model: Model object of the respective entity
    :return: JSON object and HTTP Status Code
    """
    kwargs = {}
    for key in request.args.keys():
        if key not in model.allowed_params:
            response = {
                'code': 'BAD REQUEST',
                'message': 'Query Param {} not allowed'.format(key)
            }
            return jsonify(response), 400
        kwargs[key] = request.args.get(key)
    if 'page' not in kwargs:
        kwargs['page'] = 1
    # count, records, prev_page_no, next_page_no = model.fetch_all(**kwargs)
    count, records = model.fetch_all(**kwargs)
    serialized_records = [model.serialize_record(rec) for rec in records]

    # filters = dict(request.args.deepcopy())
    # filters['page'] = next_page_no
    # next_page_url = None if not next_page_no else "{}?{}".format(request.base_url, urlencode(filters))
    # filters['page'] = prev_page_no
    # prev_page_url = None if not prev_page_no else "{}?{}".format(request.base_url, urlencode(filters))

    response = {
        'data': serialized_records,
        'count': count
        # 'next_page': next_page_url,
        # 'prev_page': prev_page_url
    }

    return jsonify(response), 200
